﻿'   Ted - Text Dumper Utility
'   Copyright (C) 2009  Rocket Science

'   This program is free software: you can redistribute it and/or modify
'   it under the terms of the GNU General Public License as published by
'   the Free Software Foundation, either version 3 of the License, or
'   (at your option) any later version.
'
'   This program is distributed in the hope that it will be useful,
'   but WITHOUT ANY WARRANTY; without even the implied warranty of
'   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'   GNU General Public License for more details.
'
'   You should have received a copy of the GNU General Public License
'   along with this program.  If not, see <http://www.gnu.org/licenses/>.
'
'   frmMain.vb - code file for main Form

Imports System.Text ' faster access to System.Text.Encoding
Public Class frmMain
    Private fileStream As IO.FileStream     ' FileStream to read data from the ROM
    Private binaryStream As IO.BinaryReader ' BinaryStream to wrap around fileStream
    Private currentProject As New Project   ' Project data
    Private currentSettings As Settings     ' Program settings

    ' Called when btnLoadText is clicked
    Private Sub btnLoadText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLoadText.Click
        Try
            ' Open ROM for read access
            fileStream = New IO.FileStream(txtPath.Text, IO.FileMode.Open, IO.FileAccess.ReadWrite, IO.FileShare.Read)
            fileStream.Position = CInt(txtStartAddress.Text) ' Set starting position according to user input
            binaryStream = New IO.BinaryReader(fileStream, currentProject.Encoding)

            ' Variables used in Do...Loop
            Dim item As New ReferenceItem()                     ' Temporary variable to store pointer and text
            Dim isHex As Boolean
            If txtEndBytes.Text.StartsWith("&H") Then
                isHex = True    ' End bytes variable
            Else
                isHex = False
            End If

            Dim byteArray() As Char
            If isHex = True Then
                ReDim byteArray(CInt((txtEndBytes.Text.Length - 2) / 2) - 1)
            Else
                ReDim byteArray(CInt(txtEndBytes.Text.Length / 2) - 1)
            End If

            Dim i, i2 As Integer
            Dim endBytes As String = txtEndBytes.Text.Replace("&H", vbNullString)
            Do
                If isHex = True Then
                    byteArray(i) = Chr(Int32.Parse(endBytes.Substring(i2, 2), Globalization.NumberStyles.AllowHexSpecifier))
                Else
                    byteArray(i) = Chr(CInt(txtEndBytes.Text.Substring(i2, 2)))
                End If
                i += 1
                i2 += 2
            Loop Until i2 >= endBytes.Length

            item.Address = CULng(fileStream.Position)           ' Store pointer address

            Do
                ' Read a character from the file
                item.Text += binaryStream.ReadChar
                ' Check for end flags
                If item.Text.Length >= txtEndBytes.Text.Length - 2 AndAlso currentProject.Encoding.GetString(currentProject.Encoding.GetBytes(item.Text.Substring(item.Text.Length - CInt(endBytes.Length / 2)))) = currentProject.Encoding.GetString(currentProject.Encoding.GetBytes(byteArray)) Then
                    ' Finish reading text and move to next line of dialogue
                    'item.Text += binaryStream.ReadChar
                    lstPointers.Items.Add(item)
                    item = New ReferenceItem()
                    While binaryStream.PeekChar = &H0
                        fileStream.Position += 1
                        If fileStream.Position >= CInt(txtEndAddress.Text) Then
                            Exit Do
                        End If
                    End While
                    item.Address = CULng(fileStream.Position)
                End If
            Loop While fileStream.Position < CInt(txtEndAddress.Text) ' Continue until end address is reached
        Catch ex As IO.FileNotFoundException
            MessageBox.Show("The file path you specified is incorrect.", "Invalid Input Entered", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch ex As InvalidCastException
            MessageBox.Show("Please ensure all the settings are correct and try again.", "Invalid Input Entered", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch ex As ArgumentOutOfRangeException
            MessageBox.Show("Please ensure all the settings are correct and try again.", "Invalid Input Entered", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch ex As OverflowException
            MessageBox.Show("Please ensure all the settings are correct and try again.", "Invalid Input Entered", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch ex As ArgumentException
            MessageBox.Show("Please specify a file to read from.", "Invalid Input Entered", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Finally
            binaryStream.Close()
        End Try
    End Sub

    ' Called when the form is closing
    Private Sub frmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ' Close filestreams if applicable
        Try
            binaryStream.Close()
            fileStream.Close()
        Catch ex As NullReferenceException ' Filestreams are not already open
        End Try
    End Sub

    'Called when the form is loaded
    Private Sub frmMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' Initialise global variables
        currentProject = New Project()
        currentProject.Encoding = System.Text.Encoding.GetEncoding("Shift_JIS")
        'MessageBox.Show(currentProject.Encoding.EncodingName)
        cbxEncodings.Items.AddRange(Encoding.GetEncodings)
        openDialog.InitialDirectory = Application.StartupPath
        saveDialog.InitialDirectory = Application.StartupPath
    End Sub

    ' Called when an item in the list box is selected
    Private Sub lstPointers_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles lstPointers.SelectedIndexChanged
        ' Display text in the rich text box
        rtbText.Text = vbNullString
        rtbText.Text = CType(lstPointers.SelectedItem, ReferenceItem).Text
    End Sub

    ' Called when the user chooses to export a Kruptar-format pointer list
    Private Sub KruptarPointerListToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles KruptarPointerListToolStripMenuItem.Click
        If Not lstPointers.Items.Count = 0 AndAlso saveDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
            ' Fetch pointers and write to a text file
            Dim strOutput As String = "cp" & currentProject.Encoding.WindowsCodePage.ToString
            For Each item As ReferenceItem In lstPointers.Items
                strOutput += vbNewLine & Hex(item.Address)
            Next
            My.Computer.FileSystem.WriteAllText(saveDialog.FileName, strOutput, False)
        ElseIf lstPointers.Items.Count = 0 Then
            MessageBox.Show("There are no lines of text to export; please load some text and try again.", "Could not export Kruptar pointers", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End If
    End Sub

    ' Called when the user chooses to show the About dialog box
    Private Sub AboutTedToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutTedToolStripMenuItem.Click
        frmAbout.ShowDialog()
    End Sub

    ' Called when the user selects an item in the combo box
    Private Sub cbxEncodings_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cbxEncodings.SelectedIndexChanged
        If cbxEncodings.SelectedIndex = 0 Then
        Else
            currentProject.Encoding = CType(cbxEncodings.SelectedItem, EncodingInfo).GetEncoding()
        End If
    End Sub

    ' Called when the user chooses the Exit menu option (shortcut: ALT+F4)
    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click
        Application.Exit()
    End Sub

    ' Called when the user chooses to open a file
    Private Sub btnOpenFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOpenFile.Click
        If openDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
            txtPath.Text = openDialog.FileName
        End If
    End Sub

    ' Called when the user chooses to export a script dump
    Private Sub RawScriptDumpToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RawScriptDumpToolStripMenuItem.Click
        If Not lstPointers.Items.Count = 0 AndAlso saveDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Dim strOutput As String
            For Each item As ReferenceItem In lstPointers.Items
                strOutput += Hex(item.Address) & " " & item.Text & vbNewLine
            Next
            My.Computer.FileSystem.WriteAllText(saveDialog.FileName, strOutput, False)
        ElseIf lstPointers.Items.Count = 0 Then
            MessageBox.Show("There are no lines of text to export; please load some text and try again.", "Could not export script dump", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End If
    End Sub
End Class
